@@ -1,5 +1,5 @@
-/*
- * 
+/**
+ * @example example.c
  * This is an example of how to use libvncserver.
  * 
  * libvncserver example
@@ -34,13 +34,13 @@
 #include <rfb/rfb.h>
 #include <rfb/keysym.h>
 
-const int bpp=4;
-int maxx=800, maxy=600;
+static const int bpp=4;
+static int maxx=800, maxy=600;
 /* TODO: odd maxx doesn't work (vncviewer bug) */
 
 /* This initializes a nice (?) background */
 
-void initBuffer(unsigned char* buffer)
+static void initBuffer(unsigned char* buffer)
 {
   int i,j;
   for(j=0;j<maxy;++j) {
@@ -63,12 +63,12 @@ typedef struct ClientData {
   int oldx,oldy;
 } ClientData;
 
-void clientgone(rfbClientPtr cl)
+static void clientgone(rfbClientPtr cl)
 {
   free(cl->clientData);
 }
 
-enum rfbNewClientAction newclient(rfbClientPtr cl)
+static enum rfbNewClientAction newclient(rfbClientPtr cl)
 {
   cl->clientData = (void*)calloc(sizeof(ClientData),1);
   cl->clientGoneHook = clientgone;
@@ -77,7 +77,7 @@ enum rfbNewClientAction newclient(rfbClientPtr cl)
 
 /* switch to new framebuffer contents */
 
-void newframebuffer(rfbScreenInfoPtr screen, int width, int height)
+static void newframebuffer(rfbScreenInfoPtr screen, int width, int height)
 {
   unsigned char *oldfb, *newfb;
 
@@ -94,7 +94,7 @@ void newframebuffer(rfbScreenInfoPtr screen, int width, int height)
 
 /* aux function to draw a line */
 
-void drawline(unsigned char* buffer,int rowstride,int bpp,int x1,int y1,int x2,int y2)
+static void drawline(unsigned char* buffer,int rowstride,int bpp,int x1,int y1,int x2,int y2)
 {
   int i,j;
   i=x1-x2; j=y1-y2;
@@ -107,7 +107,6 @@ void drawline(unsigned char* buffer,int rowstride,int bpp,int x1,int y1,int x2,i
   if(j<0) j=-j;
   if(i<j) {
     if(y1>y2) { i=y2; y2=y1; y1=i; i=x2; x2=x1; x1=i; }
-    if(y2==y1) { if(y2>0) y1--; else y2++; }
     for(j=y1;j<=y2;j++)
       for(i=0;i<bpp;i++)
 	buffer[j*rowstride+(x1+(j-y1)*(x2-x1)/(y2-y1))*bpp+i]=0xff;
@@ -121,13 +120,10 @@ void drawline(unsigned char* buffer,int rowstride,int bpp,int x1,int y1,int x2,i
     
 /* Here the pointer events are handled */
 
-void doptr(int buttonMask,int x,int y,rfbClientPtr cl)
+static void doptr(int buttonMask,int x,int y,rfbClientPtr cl)
 {
    ClientData* cd=cl->clientData;
 
-   if(cl->screen->cursorIsDrawn)
-     rfbUndrawCursor(cl->screen);
-
    if(x>=0 && y>=0 && x<maxx && y<maxy) {
       if(buttonMask) {
 	 int i,j,x1,x2,y1,y2;
@@ -135,6 +131,9 @@ void doptr(int buttonMask,int x,int y,rfbClientPtr cl)
 	 if(cd->oldButton==buttonMask) { /* draw a line */
 	    drawline((unsigned char*)cl->screen->frameBuffer,cl->screen->paddedWidthInBytes,bpp,
 		     x,y,cd->oldx,cd->oldy);
+	    x1=x; y1=y;
+	    if(x1>cd->oldx) x1++; else cd->oldx++;
+	    if(y1>cd->oldy) y1++; else cd->oldy++;
 	    rfbMarkRectAsModified(cl->screen,x,y,cd->oldx,cd->oldy);
 	 } else { /* draw a point (diameter depends on button) */
 	    int w=cl->screen->paddedWidthInBytes;
@@ -146,7 +145,7 @@ void doptr(int buttonMask,int x,int y,rfbClientPtr cl)
 	    for(i=x1*bpp;i<x2*bpp;i++)
 	      for(j=y1;j<y2;j++)
 		cl->screen->frameBuffer[j*w+i]=(char)0xff;
-	    rfbMarkRectAsModified(cl->screen,x1,y1,x2-1,y2-1);
+	    rfbMarkRectAsModified(cl->screen,x1,y1,x2,y2);
 	 }
 
 	 /* we could get a selection like that:
@@ -166,14 +165,18 @@ void doptr(int buttonMask,int x,int y,rfbClientPtr cl)
 
 /* Here the key events are handled */
 
-void dokey(rfbBool down,rfbKeySym key,rfbClientPtr cl)
+static void dokey(rfbBool down,rfbKeySym key,rfbClientPtr cl)
 {
   if(down) {
     if(key==XK_Escape)
       rfbCloseClient(cl);
+    else if(key==XK_F12)
+      /* close down server, disconnecting clients */
+      rfbShutdownServer(cl->screen,TRUE);
+    else if(key==XK_F11)
+      /* close down server, but wait for all clients to disconnect */
+      rfbShutdownServer(cl->screen,FALSE);
     else if(key==XK_Page_Up) {
-      if(cl->screen->cursorIsDrawn)
-	rfbUndrawCursor(cl->screen);
       initBuffer((unsigned char*)cl->screen->frameBuffer);
       rfbMarkRectAsModified(cl->screen,0,0,maxx,maxy);
     } else if (key == XK_Up) {
@@ -195,9 +198,7 @@ void dokey(rfbBool down,rfbKeySym key,rfbClientPtr cl)
     } else if(key>=' ' && key<0x100) {
       ClientData* cd=cl->clientData;
       int x1=cd->oldx,y1=cd->oldy,x2,y2;
-      if(cl->screen->cursorIsDrawn)
-	rfbUndrawCursor(cl->screen);
-      cd->oldx+=rfbDrawChar(cl->screen,&radonFont,cd->oldx,cd->oldy,(char)key,0x00ffffff);
+      cd->oldx+=rfbDrawCharWithClip(cl->screen,&radonFont,cd->oldx,cd->oldy,(char)key,0,0,cl->screen->width,cl->screen->height,0x00ffffff,0x00ffffff);
       rfbFontBBox(&radonFont,(char)key,&x1,&y1,&x2,&y2);
       rfbMarkRectAsModified(cl->screen,x1,y1,x2-1,y2-1);
     }
@@ -206,8 +207,10 @@ void dokey(rfbBool down,rfbKeySym key,rfbClientPtr cl)
 
 /* Example for an XCursor (foreground/background only) */
 
-int exampleXCursorWidth=9,exampleXCursorHeight=7;
-char exampleXCursor[]=
+#ifdef JUST_AN_EXAMPLE
+
+static int exampleXCursorWidth=9,exampleXCursorHeight=7;
+static char exampleXCursor[]=
   "         "
   " xx   xx "
   "  xx xx  "
@@ -216,9 +219,11 @@ char exampleXCursor[]=
   " xx   xx "
   "         ";
 
+#endif
+
 /* Example for a rich cursor (full-colour) */
 
-void MakeRichCursor(rfbScreenInfoPtr rfbScreen)
+static void MakeRichCursor(rfbScreenInfoPtr rfbScreen)
 {
   int i,j,w=32,h=32;
   rfbCursorPtr c = rfbScreen->cursor;
@@ -258,7 +263,8 @@ void MakeRichCursor(rfbScreenInfoPtr rfbScreen)
   c=rfbScreen->cursor = rfbMakeXCursor(w,h,bitmap,bitmap);
   c->xhot = 16; c->yhot = 24;
 
-  c->richSource = (char*)malloc(w*h*bpp);
+  c->richSource = (unsigned char*)malloc(w*h*bpp);
+  c->cleanupRichSource = TRUE;
   for(j=0;j<h;j++) {
     for(i=0;i<w;i++) {
       c->richSource[j*w*bpp+i*bpp+0]=i*0xff/w;
@@ -273,8 +279,9 @@ void MakeRichCursor(rfbScreenInfoPtr rfbScreen)
 
 int main(int argc,char** argv)
 {
-  rfbScreenInfoPtr rfbScreen =
-    rfbGetScreen(&argc,argv,maxx,maxy,8,3,bpp);
+  rfbScreenInfoPtr rfbScreen = rfbGetScreen(&argc,argv,maxx,maxy,8,3,bpp);
+  if(!rfbScreen)
+    return 0;
   rfbScreen->desktopName = "LibVNCServer Example";
   rfbScreen->frameBuffer = (char*)malloc(maxx*maxy*bpp);
   rfbScreen->alwaysShared = TRUE;
@@ -301,7 +308,7 @@ int main(int argc,char** argv)
 #ifdef USE_OWN_LOOP
   {
     int i;
-    for(i=0;;i++) {
+    for(i=0;rfbIsActive(rfbScreen);i++) {
       fprintf(stderr,"%d\r",i);
       rfbProcessEvents(rfbScreen,100000);
     }
@@ -318,11 +325,11 @@ int main(int argc,char** argv)
 
   /* this is the non-blocking event loop; a background thread is started */
   rfbRunEventLoop(rfbScreen,-1,TRUE);
+  fprintf(stderr, "Running background loop...\n");
   /* now we could do some cool things like rendering in idle time */
   while(1) sleep(5); /* render(); */
 #endif /* BACKGROUND_LOOP */
 
-  rfbFreeCursor(rfbScreen->cursor);
   free(rfbScreen->frameBuffer);
   rfbScreenCleanup(rfbScreen);
 
