@@ -145,6 +145,11 @@ rfbUnregisterProtocolExtension(rfbProtocolExtension* extension)
 
 rfbProtocolExtension* rfbGetExtensionIterator()
 {
+	if (! extMutex_initialized) {
+		INIT_MUTEX(extMutex);
+		extMutex_initialized = 1;
+	}
+
 	LOCK(extMutex);
 	return rfbExtensionHead;
 }
@@ -242,7 +247,7 @@ rfbDefaultLog(const char *format, ...)
 
     time(&log_clock);
     strftime(buf, 255, "%d/%m/%Y %X ", localtime(&log_clock));
-    fprintf(stderr,buf);
+    fprintf(stderr, "%s", buf);
 
     vfprintf(stderr, format, args);
     fflush(stderr);
@@ -444,23 +449,34 @@ clientOutput(void *data)
     while (1) {
         haveUpdate = false;
         while (!haveUpdate) {
-            if (cl->sock == -1) {
-                /* Client has disconnected. */
-                return NULL;
-            }
-	    LOCK(cl->updateMutex);
-	    haveUpdate = FB_UPDATE_PENDING(cl);
-	    if(!haveUpdate) {
-		updateRegion = sraRgnCreateRgn(cl->modifiedRegion);
-		haveUpdate = sraRgnAnd(updateRegion,cl->requestedRegion);
-		sraRgnDestroy(updateRegion);
-	    }
-	    UNLOCK(cl->updateMutex);
+		if (cl->sock == -1) {
+			/* Client has disconnected. */
+			return NULL;
+		}
+		if (cl->state != RFB_NORMAL || cl->onHold) {
+			/* just sleep until things get normal */
+			usleep(cl->screen->deferUpdateTime * 1000);
+			continue;
+		}
 
-            if (!haveUpdate) {
-                WAIT(cl->updateCond, cl->updateMutex);
-		UNLOCK(cl->updateMutex); /* we really needn't lock now. */
-            }
+		LOCK(cl->updateMutex);
+
+		if (sraRgnEmpty(cl->requestedRegion)) {
+			; /* always require a FB Update Request (otherwise can crash.) */
+		} else {
+			haveUpdate = FB_UPDATE_PENDING(cl);
+			if(!haveUpdate) {
+				updateRegion = sraRgnCreateRgn(cl->modifiedRegion);
+				haveUpdate   = sraRgnAnd(updateRegion,cl->requestedRegion);
+				sraRgnDestroy(updateRegion);
+			}
+		}
+
+		if (!haveUpdate) {
+			WAIT(cl->updateCond, cl->updateMutex);
+		}
+
+		UNLOCK(cl->updateMutex);
         }
         
         /* OK, now, to save bandwidth, wait a little while for more
@@ -477,7 +493,9 @@ clientOutput(void *data)
 
         /* Now actually send the update. */
 	rfbIncrClientRef(cl);
+        LOCK(cl->sendMutex);
         rfbSendFramebufferUpdate(cl, updateRegion);
+        UNLOCK(cl->sendMutex);
 	rfbDecrClientRef(cl);
 
 	sraRgnDestroy(updateRegion);
@@ -499,6 +517,11 @@ clientInput(void *data)
 	struct timeval tv;
 	int n;
 
+	if (cl->sock == -1) {
+	  /* Client has disconnected. */
+            break;
+        }
+
 	FD_ZERO(&rfds);
 	FD_SET(cl->sock, &rfds);
 	FD_ZERO(&efds);
@@ -528,11 +551,6 @@ clientInput(void *data)
 
         if (FD_ISSET(cl->sock, &rfds) || FD_ISSET(cl->sock, &efds))
             rfbProcessClientMessage(cl);
-
-        if (cl->sock == -1) {
-            /* Client has disconnected. */
-            break;
-        }
     }
 
     /* Get rid of the output thread. */
@@ -881,7 +899,9 @@ rfbScreenInfoPtr rfbGetScreen(int* argc,char** argv,
    screen->setTranslateFunction = rfbSetTranslateFunction;
    screen->newClientHook = rfbDefaultNewClientHook;
    screen->displayHook = NULL;
+   screen->displayFinishedHook = NULL;
    screen->getKeyboardLedStateHook = NULL;
+   screen->xvpHook = NULL;
 
    /* initialize client list and iterator mutex */
    rfbClientListInit(screen);
@@ -983,9 +1003,6 @@ void rfbScreenCleanup(rfbScreenInfoPtr screen)
   if(screen->cursor && screen->cursor->cleanup)
     rfbFreeCursor(screen->cursor);
 
-  rfbRRECleanup(screen);
-  rfbCoRRECleanup(screen);
-  rfbUltraCleanup(screen);
 #ifdef LIBVNCSERVER_HAVE_LIBZ
   rfbZlibCleanup(screen);
 #ifdef LIBVNCSERVER_HAVE_LIBJPEG
@@ -1064,9 +1081,6 @@ rfbProcessEvents(rfbScreenInfoPtr screen,long usec)
 
   rfbCheckFds(screen,usec);
   rfbHttpCheckFds(screen);
-#ifdef CORBA
-  corbaCheckFds(screen);
-#endif
 
   i = rfbGetClientIteratorWithClosed(screen);
   cl=rfbClientIteratorHead(i);
